# 
#  This file has to be sourced before activating AUTO if you are using
#  a 'sh' compatible shell, such as sh, bash, ksh, or ash.
#
AUTO_DIR=$HOME/auto/07p
PATH=$AUTO_DIR/cmds:$AUTO_DIR/bin:$PATH
# the following is an example (to be uncommented) for Windows+MSYS:
#PATH="/c/Python27:/bin:/c/Program Files/gfortran/bin:$PATH"
export AUTO_DIR
export PATH
#
# DON'T ADD ANYTHING AFTER THIS LINE
#
