C plautcom.f
      COMMON /PLTCOM/ TMINVX,TMINVY,TMAXVX,TMAXVY,TREALX,
     1 TREALY,TIMAGX,TIMAGY,TRCOSF,TRSINF,TRSCAL,TRFACX,
     2 TRFACY,TRPAR1,TRPAR2,TRPAR3,TRPAR4,TRPAR5,TRPAR6,
     3 KMOFLG(8),KPAD2,KBAUDR,KGNFLG,KGRAFL,KHOMEY,KKMODE,
     4 KHORSZ,KVERSZ,KTBLSZ,KSIZEF,KLMRGN,KRMRGN,KFACTR,
     5 KTERM,KLINE,KZAXIS,KBEAMX,KBEAMY,KMOVEF,KPCHAR(5),
     6 KDASHT,KMINSX,KMINSY,KMAXSX,KMAXSY,KEYCON,KINLFT,
     7 KOTLFT,KUNIT

